param([String]$inputfile)

$global:confirm_setinit = 0;
$global:confirm_sid = 0;
$global:setinit = "";

#
# Main function.
#
Function Main {
    # Opens file dialog box if $filename is not passed in as an argument.
    if (!$inputfile) {
        Write-Host -NoNewline "Please choose a license file: "
        $inputfile = Get-FileName
        Write-Host $inputfile
    }
    
    $dir = Join-Path $env:ALLUSERSPROFILE -ChildPath "SAS\ApplySASLicense"
    $result = Test-Path $dir
    if ($result -eq $FALSE) {
        New-Item $dir -ItemType Directory
    }
    Set-License $inputfile $dir
}

#
# Throw error if $state does not match what we found while parsing.
#
Function Check-State($state, $max) {
    if ($state -gt $max) 
    {
        Write-Host "ERROR: Bad license file."
        exit 1
    }
}

#
# This method returns the location of sashome. We are assuming that this script resides
# in $sashome/Utilities/bin. We will need to go up two directories to return $sashome.
#
Function Find-Sasroot() {
    return Split-Path (Split-Path $PSScriptRoot -Parent) -Parent
}

#
# Opens file dialog and filters file listings via *.txt. 
#
Function Get-FileName() {       
    Add-Type -AssemblyName System.Windows.Forms
    $OpenFileDialog = New-Object System.Windows.Forms.OpenFileDialog
    # Initial directory will be set to desktop of the userprofile currently running script.
    $OpenFileDialog.initialDirectory = Join-Path $env:USERPROFILE -ChildPath "Desktop"
    $OpenFileDialog.filter = "TXT (*.txt)| *.txt"
    #$OpenFileDialog.ShowHelp = $true
    $OpenFileDialog.ShowDialog() | Out-Null   
    $OpenFileDialog.filename
}

#
# Extracts content from global setinit variable to the setinit.sss file in the 
# ProgramData directory.
#
Function Create-SetinitFile($dir) {    
    if ($global:confirm_setinit -eq 1) {
        $setinitfile = Join-Path $dir -ChildPath "setinit.sss"
        $global:setinit | Out-File $setinitfile
        return $setinitfile
    }
}

#
# This method runs sas.exe command, with arguments, to apply the license.
#
Function Set-License($setinit, $profile) {
    $dir
    # SASHome location relative to script.
    $sasroot = Find-Sasroot
    $binpath = Join-Path $sasroot -ChildPath "sas.exe"
    # ProgramData folder will contain logs and SAS catalog files generated from applying license.
    $logfile = Join-Path $profile -ChildPath "setinit.log"
    $arguments =  "-setinit -sysin ""$setinit"" -log ""$profile\setinit.log"" -nodms -noterminal -noautoexec -nodate " 
    $arguments += "-nostimer -unbuflog -sasuser ""$profile"" -icon -nosplash -noprngetlist -msg !SASROOT/sasmsg -config "
    $arguments += """$sasroot\sasv9.cfg"""
    Write-Verbose "Applying license: '$binPath $arguments'"
    $process = Start-Process -FilePath $binpath -ArgumentList $arguments -PassThru -Wait -NoNewWindow
    if ($process.ExitCode -ne 0) {
        Write-Error "ERROR: Apply SAS license file failed with an exit code of $($process.ExitCode). See log file: $logfile."
        exit
    }
    Write-Host "SAS license file has been applied."
}

Main
