#This class represents the structure of a Viya deployment. Based on an installdir,
#this class can calculate that paths to various important directories.
class ViyaHierarchy {
    [string] $installRoot
    [string] $deploymentId
    [string] $deploymentControllerRoot

    ViyaHierarchy([string] $installRoot, [string] $deploymentId, [string] $deploymentControllerRoot) {
        $this.installRoot = $installRoot
        $this.deploymentId = $deploymentId
        $this.deploymentControllerRoot = $deploymentControllerRoot
    }

    [string] GetInstallDir() {
        return $this.installRoot
    }

    #fixme: it's more efficient to lazy load the paths rather than
    #recalculating them every time.
    [string] GetSasHomeDir() {
        $pieces = @($this.GetInstallDir(),$this.deploymentId)
        return $this.joinPaths($pieces)
    }

    [string] GetLibraryDir() {
        $pieces = @($this.GetSasHomeDir(), "Utilities", "bin")
        return $this.joinPaths($pieces)
    }

    [string] GetModulesDir() {
        $pieces = @($this.GetSasHomeDir(), "Utilities", "DeploymentTools")
        return $this.joinPaths($pieces)
    }

    [string] GetConfigDir() {
        $pieces = @($env:ProgramData, "SAS", $this.deploymentId)
        return $this.joinPaths($pieces)
    }

    [string] GetPublicSasDir() {
        $pieces = @($env:PUBLIC, "SAS", $this.deploymentId)
        return $this.joinPaths($pieces)
    }

    [string] GetInstallLogDir() {
        $pieces = @($this.GetPublicSasDir(), "logs", "install")
        return $this.joinPaths($pieces)
    }

    [string] GetUninstallDataFile(){
        $pieces = @($this.GetLibraryDir(), "uninstall", "uninstall_data.xml")
        return $this.joinPaths($pieces)
    }

    #Resolves the relative path to the root of the install scripts dir.
    [string] GetDeploymentToolsRoot() {
        #This is the least nasty way I could think of to do the pathing here.
        $pieces = @($this.deploymentControllerRoot, "..", "..")
        $toolRoot = $this.joinPaths($pieces)
        return Resolve-Path $toolRoot
    }

    #Resolves the relative path to the module library.
    [string] GetDeploymentModuleLibrary() {
        $moduleLibrary = Join-Path $this.deploymentControllerRoot ".."
        return Resolve-Path $moduleLibrary
    }

    hidden [string] joinPaths([System.Array] $pieces){
        $path = ''
        foreach($piece in $pieces){
            if($path -eq ''){
                $path = $piece
                continue
            }
            $path = Join-Path $path $piece
        }

        return $path
    }
}