#Functions for handling certificate operations. The object is to allow for easier mocking.

function ImportCertificateAuthority {
    param(
        [string] $caFile
    )
    if(!(Test-Path $caFile)){
        throw "Certificate file '$caFile' does not exist. Certificate could not be imported."
    }

    $certChain = New-Object System.Security.Cryptography.X509Certificates.X509Certificate2Collection
    $certChain.Import($caFile)

    foreach($cert in $certChain) {

        #if self signed, put it in the root CA store
        if($cert.Subject -eq $cert.Issuer) {
            StoreCertificate $cert ([System.Security.Cryptography.X509Certificates.StoreName]::Root)
        } else {
            StoreCertificate $cert ([System.Security.Cryptography.X509Certificates.StoreName]::CertificateAuthority)
        }
    }
}

function RemoveCertificateAuthority {
    param(
        [string] $caFile
    )

    if(!(Test-Path $caFile)){
        throw "Certificate file '$caFile' does not exist. Certificate could not be removed."
    }

    $certChain = New-Object System.Security.Cryptography.X509Certificates.X509Certificate2Collection
    $certChain.Import($caFile)

    foreach($cert in $certChain) {
        if($cert.Subject -eq $cert.Issuer) {
            RemoveCertificate $cert ([System.Security.Cryptography.X509Certificates.StoreName]::Root)
        } else {
            RemoveCertificate $cert ([System.Security.Cryptography.X509Certificates.StoreName]::CertificateAuthority)
        }
    }
}

function StoreCertificate {
    param(
        [System.Security.Cryptography.X509Certificates.X509Certificate2] $certificate,
        [System.Security.Cryptography.X509Certificates.StoreName] $storeName
    )

    $certStore = New-Object System.Security.Cryptography.X509Certificates.X509Store ($storeName, ([System.Security.Cryptography.X509Certificates.StoreLocation]::LocalMachine))
    $certStore.Open([System.Security.Cryptography.X509Certificates.OpenFlags]::ReadWrite)
    $certStore.Add($certificate)
    $certStore.Close()
}

function RemoveCertificate {
    param(
        [System.Security.Cryptography.X509Certificates.X509Certificate2] $certificate,
        [System.Security.Cryptography.X509Certificates.StoreName] $storeName
    )

    $certStore = New-Object System.Security.Cryptography.X509Certificates.X509Store ($storeName, ([System.Security.Cryptography.X509Certificates.StoreLocation]::LocalMachine))
    $certStore.Open([System.Security.Cryptography.X509Certificates.OpenFlags]::ReadWrite)
    $certStore.Remove($certificate)
    $certStore.Close()
}
