#
#Helper functions for repositories.
#

function CompileRepoData() {
    param(
        [string] $baseUrl,
        [System.Security.Cryptography.X509Certificates.X509Certificate2] $entitlementCert,
        [string[]] $repoPaths
    )

    # Create a yum repo object for each repo in the control file.
    $yumRepos = @{}
    foreach ($repoPath in $repoPaths) {
        $yumRepo = BuildYumRepo $baseUrl $repoPath $entitlementCert
        $yumRepos[$repoPath] = $yumRepo
    }

    return $yumRepos
}

function BuildYumRepo() {
    param(
        [string] $baseUrl,
        [string] $subUrl,
        [System.Security.Cryptography.X509Certificates.X509Certificate2] $entitlementCert
    )

    $repoUrl = JoinUrlPieces @($baseUrl, $subUrl)
    $repomdUrl = JoinUrlPieces @($repoUrl, "repodata/repomd.xml")
    $repoMd = GetRepomdData $repomdUrl $entitlementCert
    $primaryUrl = JoinUrlPieces @($repoUrl, $repoMd.GetPrimaryFileLocation())
    $primaryMd = GetPrimaryData $primaryUrl $entitlementCert

    $yumRepo = [Repository]::new($baseUrl, $subUrl, $repoMd, $primaryMd)

    return $yumRepo
}

function GetEntitlementData {
    param(
        [string] $baseUrl,
        [System.Security.Cryptography.X509Certificates.X509Certificate2] $entitlementCert
    )

    $entUrl = JoinUrlPieces @($baseUrl, 'entitlements.json')
    $data = DownloadControlData $entUrl $entitlementCert
    $controlData = [EntitlementData]::new($data)

    return $controlData
}

function GetRepomdData {
    param(
        [string] $url,
        [System.Security.Cryptography.X509Certificates.X509Certificate2] $entitlementCert
    )

    [xml] $results = DownloadRepoMD $url $entitlementCert
    $metadata = [RepoMd]::new($results)

    return $metadata
}

function GetPrimaryData {
    param(
        [string] $url,
        [System.Security.Cryptography.X509Certificates.X509Certificate2] $entitlementCert
    )

    [xml] $data = DownloadPrimary $url $entitlementCert
    $primary = [Primary]::new($data)

    return $primary
}