The SasEntitlement module is use to download packages from our pulp instances
to the customer's systems given the entitlement and CA certificates in an SOE.


Usage is as follows:
```powershell
$pfxCAChain = "C:\temp\entitlements\ca_chain.pfx"
$entitlementCert = "C:\temp\entitlements\entitlement_certificate.pfx" 
$downloadDir = "C:\\temp\\entitlements\\packages"

Import-Module -Name SasEntitlement -Force

Get-AllEntitlement -downloadDir $downloadDir -rootCAPath $sasRootCA -entitlementCertPath $entitlementCert -Verbose
Get-Entitlement -downloadDir $downloadDir -rootCAPath $sasRootCA -entitlementCertPath $entitlementCert -packageName "sas-tkcore" -Verbose
Remove-Entitlement -downloadDir $downloadDir -rootCAPath $sasRootCA -Verbose
Get-EntitlementFromUrl -Verbose -downloadDir $downloadDir -EntitlementCert $entCert `
   -rootCAPath $cacert -url https://ses.sas.download/ses/win/esp/esp-100.0.0-win-latest/msiesp-1.0.0.57.msi

#Note: $data is an array here. Use '@()' to force the data to always return as an array.
#Otherwise, lists of 1 will be returned as a single object.
$data = @(Get-PackageMetadata -packageName "msiesp" -rootCAPath $cacert -entitlementCertPath $entCert)
$data[0].UpgradeCode
$data[0].ProductCode
$data[0].Url
```

* `Get-AllEntitlement`, as its name implies, will retrieve all packages that the
entitlement certificate has access to.
* `Get-Entitlement` will retrieve a single entitlement when given its name.
In both cases, if there is more than one version of a package, the latest is
taken.
* `Remove-Entitlement` will delete the contents of whatever the downloadDir is,
and remove the installed certificates from the certificate store.
* `Get-EntitlementFromUrl` Grabs a packaged directly from the download URL specified.
This assumes you know the download URL in advance.
* `Get-PackageMetadata` Retrieves the UpgradeCode, ProductCode, and download URL 
for the latest version of the package name specified. This function returns
an array of PSCustomObject to account for the fact that we could have a 
package of the same name existing in multiple repos.