#Fuctions for performing preflight verification of nodes
function Get-PSVersion{
    param(
        [Parameter(Mandatory=$true)]
        [string] $remoteHost
    )

    $versionTable = Invoke-Command -Computername $remoteHost -Scriptblock {$PSVersionTable}
    [string]$major = $versionTable.psversion.Major
    [string]$minor = $versionTable.psversion.Minor
    [string]$out = $major + '.' + $minor
    [decimal]$remoteVersion = [convert]::ToDouble($out)
    return $remoteVersion
}

function Assert-ValidPSVersion{
    param(
        [Parameter(Mandatory=$true)]
        [decimal] $minVersion = 5.0,
        [Parameter(Mandatory=$true)]
        [array] $nodes
    )

    [boolean] $preflightPassed = $true
    [string] $errorMessage = "The following machines did not meet the PowerShell version requirement of $($minVersion.ToString("0.0")):"
    foreach ($node in $nodes) {
        [string] $nodeName = $node.NodeName
        [decimal] $nodeVersion = Get-PSVersion $nodeName

        if ( $nodeVersion -lt $minVersion){
            $preflightPassed = $false
            $errorMessage = "$($errorMessage) $($nodeName)"
        }
    }
    if ($preflightPassed -eq $false){
        throw $errorMessage
    }
}

