#wrapper functions below are used for mocking calls to the registry, since we can't mock
#the .NET functions directly.
#wrapper function. Returns a list of strings
function GetSubKeysFromPath {
    param(
        [string] $keyPath
    )

    $names = $null

    try {
        $key = [Microsoft.Win32.Registry]::LocalMachine.OpenSubKey($keyPath)

        if ($null -ne $Key) {
            $names = $key.GetSubKeyNames()
        }
    }
    finally {
        if ($null -ne $key) {
            $key.Dispose()
        }
    }

    return $names
}

#returns a custom object of product info
function GetProductInfo {
    param(
        [string] $baseKeyPath,
        [string] $productKeyName
    )

    $regInfo = $null
    $fullPath = Join-Path $baseKeyPath $productKeyName
    $key = [Microsoft.Win32.Registry]::LocalMachine.OpenSubKey($fullPath)

    if ($null -ne $key) {
        try {
            $regInfo = New-Object -TypeName psobject -Property @{
                ProductName     = $key.GetValue("ProductName")
                Version         = $key.GetValue("Version")
                InstallLocation = $key.GetValue("InstallLocation")
                ProductCode     = $key.GetValue("ProductCode")
                UpgradeCode     = $key.GetValue("UpgradeCode")
            }
        }
        finally {
            $key.Dispose()
        }
    }


    return $regInfo
}

#Reads product info from the registry given the key to where sas's
#product data are stored and returns a list of custom objects
function ReadProductInfo {
    param(
        [string] $baseKeyPath,
        [string[]] $productNames
    )

    $installedProducts = New-Object -TypeName System.Collections.ArrayList

    foreach ($name in $productNames) {
        $info = GetProductInfo $baseKeyPath $name

        #We're rolling with the assumption only sas products
        #will be installed to this location with these keys.
        if ($null -ne $info -and
            $null -ne $info.ProductName -and
            $null -ne $info.ProductCode) {
            $installedProducts.Add($info) | Out-Null
        }
    }

    return $installedProducts
}