# Write uninstall data to a file to later be consumed at uninstall.
function WriteUninstallData {
    param(
        [string] $uninstallDataFile,
        [hashtable] $uninstallData
    )

    $dataDir = Split-Path $uninstallDataFile -Parent
    $dirExists = Test-Path $dataDir
    if ($dirExists -eq $false) {
        New-Item -Path $dataDir -ItemType Directory -Force
    }

    $uninstallData | Export-Clixml -Path $uninstallDataFile
}

# Add currently installed products to a hash of all installed products and returns the resulting hash.
function MergeUninstallData {
    param(
        [hashtable] $allProducts,
        [hashtable] $productsToAdd
    )

    foreach ($key in $productsToAdd.Keys) {
        if ($allProducts.ContainsKey($key)) {
            $allProducts.Remove($key)
        }
    }

    $allProducts = $allProducts + $productsToAdd
    return $allProducts
}

# Returns a hash consisting of the product code and unit name for all the products installed.
function NewUninstallData {
    param(
        [System.Array] $products
    )

    [hashtable] $uninstallData = @{}
    foreach ($product in $products) {
        [hashtable] $data = @{}
        #store all metadata property values in install data.
        $product.psobject.properties | ForEach-Object { $data[$_.Name] = $_.Value }
        $uninstallData.Add($product.TwelveByte, $data)
    }
    return $uninstallData
}
